require "./test/test_helper"

class ShippingParserTest < MiniTest::Unit::TestCase
  attr_reader :parser

  def test_run_returns_a_parser
    assert_kind_of(Reelagram::Mail::Parsers::ShippingParser, parser.run("foo"))
  end

  def test_order_number_returns_the_order_number
    parsed_string = parser.run(data)
    assert_equal("7780000298", parsed_string.order_number)
  end

  def test_carrier_returns_the_carrier
    parsed_string = parser.run(data)
    assert_equal("United States Postal Service", parsed_string.carrier)
  end

  def test_tracking_number_returns_the_tracking_number
    parsed_string = parser.run(data)
    assert_equal("92789999982943513015003089", parsed_string.tracking_number)
  end

  def test_tracking_link_returns_the_tracking_link
    parsed_string = parser.run(data)
    assert_equal("http://wwwapps.ups.com/WebTracking/track?HTMLVersion=5.0&loc=en_US&Requester=UPSHome&WBPM_lid=homepage%2Fct1.html_pnl_trk&trackNums=92789999982943513015003089&track.x=Track target=", parsed_string.tracking_link)
  end

  def test_when_there_is_no_shipping_information_link_is_unavailable
    parsed_string = parser.run(no_shipping_data)
    assert_equal(nil, parsed_string.tracking_link)
  end

  def test_when_there_is_no_shipping_information_carrier_is_unavailable
    parsed_string = parser.run(no_shipping_data)
    assert_equal("Unavailable", parsed_string.carrier)
  end

  def test_when_there_is_no_shipping_information_tracking_number_is_unavailable
    parsed_string = parser.run(no_shipping_data)
    assert_equal("Unavailable", parsed_string.tracking_number)
  end

  def test_reset_sets_variables_to_nil
    parser.reset!
    assert_nil(parser.instance_variable_get("@shipping_info_table"))
    assert_nil(parser.instance_variable_get("@row"))
  end

  def test_parses_heroku_email
    parser.reset!
    parser.run heroku_style_email
    assert_equal("7780000265", parser.order_number)
    assert_equal("92748999982943513015002409", parser.tracking_number)
    assert_equal("http://wwwapps.ups.com/WebTracking/track?HTMLVersion=5.0&loc=en_US&Requester=UPSHome&WBPM_lid=homepage%2Fct1.html_pnl_trk&trackNums=92748999982943513015002409&track.x=Track target=", parser.tracking_link)
    assert_equal("United States Postal Service", parser.carrier)
  end

  private

  def parser
    @parser ||= Reelagram::Mail::Parsers::ShippingParser.new
  end

  def data
    FixtureHelper.fixture("email_body.txt")
  end

  def no_shipping_data
    ""
  end

  def heroku_style_email
    FixtureHelper.fixture("heroku_email.txt")
  end
end
